{ ##
  @FILE                     PJColourPopupMenuEd.pas
  @COMMENTS                 Component editor for TPJColourPopupMenu.
  @PROJECT_NAME             Colour Popup Menu
  @PROJECT_DESC             A component that provides a popup menu that displays
                            menu items for the sixteen common colours in a
                            traditional menu style or as a colour palette. A
                            component editor and 16-colour property editor are
                            also included.
  @DEPENDENCIES             Requires TPJColourPopupMenu component v1.0
  @AUTHOR                   Peter Johnson, LLANARTH, Ceredigion, Wales, UK
  @EMAIL                    peter.johnson@openlink.org
  @WEBSITE                  http://www.pjsoft.contactbox.co.uk/
  @COPYRIGHT                 Peter D Johnson, 2001.
  @LEGAL_NOTICE             This component, related component and property
                            editors and source code are placed in the public
                            domain. They may be freely copied and circulated on
                            a not-for-profit basis providing that the code is
                            unmodified and this notice and information about the
                            author and his copyright remains attached to the
                            source code.
  @HISTORY(
    @REVISION(
      @VERSION              1.0
      @DATE                 06/05/2001
      @COMMENTS             Original version.
    )
  )
}


unit PJColourPopupMenuEd;

interface


uses
  // Delphi
  Windows, Menus, StdCtrls, ComCtrls, Buttons, Controls, ExtCtrls, Classes,
  Graphics, Forms, DsgnIntf,
  // PJSoft
  PJColourPopupMenu;


type

  {Component editor that adds a menu option to TPJColourPopupMenu components to
  display the colour menu designer. Double-clicking the components also brings
  up the menu designer}
  TPJColourPopupMenuCE = class(TComponentEditor)
  protected
    procedure ExecuteEditor;
      {Displays editor dialogue box and performs required updates if user OKs}
  public
    procedure ExecuteVerb(Index: Integer); override;
      {Called when user chooses our new edit menu option added to component's
      pop-up menu: displays dialogue box}
    function GetVerb(Index: Integer): string; override;
      {Returns caption of menu item added to component's pop-up menu: here it is
      for the colour menu designer}
    function GetVerbCount: Integer; override;
      {Returns number of items added to component's pop-up menu: 1 here}
  end;


  {The TPJColourPopupMenu component editor form}
  TFmPJColourPopupMenuCE = class(TForm)
    btnCancel: TButton;
    btnHelp: TButton;
    btnOK: TButton;
    btnTest: TSpeedButton;
    bvlSep: TBevel;
    chkCheckColour: TCheckBox;
    cmbDisplayStyle: TComboBox;
    cmbMenuCols: TComboBox;
    edBmpHeight: TEdit;
    edBmpWidth: TEdit;
    lblBmpHeight: TLabel;
    lblBmpWidth: TLabel;
    lblDisplayStyle: TLabel;
    lblMenuCols: TLabel;
    lblTestBtn: TLabel;
    mnuTest: TPJColourPopupMenu;
    pnlTest: TPanel;
    udBmpHeight: TUpDown;
    udBmpWidth: TUpDown;
    procedure btnHelpClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnTestMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure chkCheckColourClick(Sender: TObject);
    procedure cmbDisplayStyleChange(Sender: TObject);
    procedure cmbMenuColsChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure mnuTestSelectColour(Sender: TObject);
    procedure udBmpHeightClick(Sender: TObject; Button: TUDBtnType);
    procedure udBmpWidthClick(Sender: TObject; Button: TUDBtnType);
  private
    fComponent: TPJColourPopupMenu;
      {Reference to component being edited}
    procedure UpdateControls;
      {Updates enabled property of controls that depend on state of others}
    procedure UpdateColour;
      {Updates colour displayed on test button}
  public
    constructor CreateEx(AOwner: TComponent; Component: TPJColourPopupMenu);
      {Extended constructor: records a reference to the component being edited}
    constructor Create(AOwner: TComponent); override;
      {Override of standard contrcutor that reaises exception: it is not
      permitted to construct this form in this manner}
  end;


procedure Register;
  {Delphi registration routine}


implementation

uses
  // Delphi
  SysUtils;

{$R *.DFM}


procedure Register;
  {Registers the property editor and component editor with Delphi}
begin
  // Register component editor
  RegisterComponentEditor(TPJColourPopupMenu, TPJColourPopupMenuCE);
end;

{ TPJColourPopupMenuCE }

procedure TPJColourPopupMenuCE.ExecuteEditor;
  {Displays editor dialogue box and performs required updates if user OKs}
var
  Dlg: TFmPJColourPopupMenuCE;  // editor dlg box instance
begin
  // Create dialogue box, passing reference to component
  Dlg := TFmPJColourPopupMenuCE.CreateEx(Application,
    Component as TPJColourPopupMenu);
  try
    // Display dlg box
    if Dlg.ShowModal = mrOK then
      // User OK'd: tell designer things have changed
      Designer.Modified;
  finally
    Dlg.Free;
  end;
end;

procedure TPJColourPopupMenuCE.ExecuteVerb(Index: Integer);
  {Called when user chooses our new edit menu option added to component's pop-up
  menu: displays dialogue box}
begin
  if Index = 0 then
    ExecuteEditor;
end;

function TPJColourPopupMenuCE.GetVerb(Index: Integer): string;
  {Returns caption of menu item added to component's pop-up menu: here it is for
  the colour menu designer}
begin
  Result := 'Colour menu &designer...';
end;

function TPJColourPopupMenuCE.GetVerbCount: Integer;
  {Returns number of items added to component's pop-up menu: 1 here}
begin
  Result := 1;
end;

{ TFmPJColourPopupMenuCE }

procedure TFmPJColourPopupMenuCE.btnHelpClick(Sender: TObject);
  {Display required help page when help button pressed}
var
  Keyword: string;  // Required topic K Keyword
begin
  Keyword := 'Colour menu designer';
  Application.HelpCommand(HELP_KEY, Cardinal(PChar(Keyword)));
end;

procedure TFmPJColourPopupMenuCE.btnOKClick(Sender: TObject);
  {User has OK'd: update component's properties according to changes}
begin
  fComponent.SelectedColour := mnuTest.SelectedColour;
  fComponent.DisplayStyle := mnuTest.DisplayStyle;
  fComponent.CheckSelectedColour := mnuTest.CheckSelectedColour;
  fComponent.MenuColumns := mnuTest.MenuColumns;
  fComponent.BitmapHeight := mnuTest.BitmapHeight;
  fComponent.BitmapWidth := mnuTest.BitmapWidth;
end;

procedure TFmPJColourPopupMenuCE.btnTestMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  {Popup the test menu}
var
  PopupPos: TPoint; // where menu is to pop-up
begin
  PopupPos := ClientToScreen(Point(btnTest.Left, btnTest.Top + btnTest.Height));
  mnuTest.Popup(PopupPos.X, PopupPos.Y);
end;

procedure TFmPJColourPopupMenuCE.chkCheckColourClick(Sender: TObject);
  {Update popup menu's CheckSelectedColour property per state of check box}
begin
  mnuTest.CheckSelectedColour := chkCheckColour.Checked;
end;

procedure TFmPJColourPopupMenuCE.cmbDisplayStyleChange(Sender: TObject);
  {Update popup menu's DisplayStyle property according to value selected in
  drop down list. Also update state of controls that depend on value of this
  property}
begin
  mnuTest.DisplayStyle := TPJColourMenuDisplayStyle(cmbDisplayStyle.ItemIndex);
  UpdateControls;
end;

procedure TFmPJColourPopupMenuCE.cmbMenuColsChange(Sender: TObject);
  {Update popup menu's MenuCols property per state of check box}
begin
  mnuTest.MenuColumns := TPJColourMenuColumns(cmbMenuCols.ItemIndex);
end;

constructor TFmPJColourPopupMenuCE.Create(AOwner: TComponent);
  {Override of standard contrcutor that reaises exception: it is not permitted
  to construct this form in this manner}
begin
  raise Exception.Create(
    'Must use CreateEx constructor for TFmPJColourPopupMenuCE');
end;

constructor TFmPJColourPopupMenuCE.CreateEx(AOwner: TComponent;
  Component: TPJColourPopupMenu);
  {Extended constructor: records a reference to the component being edited}
begin
  // Record reference to component being edited
  fComponent := Component;
  inherited Create(AOwner);
end;

procedure TFmPJColourPopupMenuCE.FormCreate(Sender: TObject);
  {Set form's control's to reflect properties of component we're editing}
begin
  // Copy relevant properties of component to be edited to private component
  mnuTest.SelectedColour := fComponent.SelectedColour;
  mnuTest.DisplayStyle := fComponent.DisplayStyle;
  mnuTest.CheckSelectedColour := fComponent.CheckSelectedColour;
  mnuTest.MenuColumns := fComponent.MenuColumns;
  mnuTest.BitmapHeight := fComponent.BitmapHeight;
  mnuTest.BitmapWidth := fComponent.BitmapWidth;
  // Update form's controls to reflect required properties
  cmbDisplayStyle.ItemIndex := Ord(fComponent.DisplayStyle);
  chkCheckColour.Checked := fComponent.CheckSelectedColour;
  cmbMenuCols.ItemIndex := Ord(fComponent.MenuColumns);
  udBmpHeight.Position := fComponent.BitmapHeight;
  udBmpWidth.Position := fComponent.BitmapWidth;
  UpdateControls;
  UpdateColour;
end;

procedure TFmPJColourPopupMenuCE.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
  {Check for F1 key and call help if detected}
begin
  if (Key = VK_F1) and (Shift = []) then
    btnHelp.Click;
end;

procedure TFmPJColourPopupMenuCE.mnuTestSelectColour(Sender: TObject);
  {Colour choise event handler for test popup menu: update colour displayed on
  test button according to current colour menu selection}
begin
  UpdateColour;
end;

procedure TFmPJColourPopupMenuCE.udBmpHeightClick(Sender: TObject;
  Button: TUDBtnType);
  {Update test menu's BitmapHeight property according to current value of
  up-down control}
begin
  mnuTest.BitmapHeight := udBmpHeight.Position;
end;

procedure TFmPJColourPopupMenuCE.udBmpWidthClick(Sender: TObject;
  Button: TUDBtnType);
  {Update test menu's BitmapWidth property according to current value of
  up-down control}
begin
  mnuTest.BitmapWidth := udBmpWidth.Position;
end;

procedure TFmPJColourPopupMenuCE.UpdateColour;
  {Updates colour displayed on test button}
const
  cBtnBmpSize = 14;   // size of bitmap
var
  Bmp: TBitmap;       // the bitmap to be assigned to button
  BmpColour: TColor;  // colour of bitmap to display
  BmpRect: TRect;     // rectangle to display bitmap in
begin
  Bmp := TBitmap.Create;
  try
    // Set bitmap's size
    Bmp.Width := cBtnBmpSize;
    Bmp.Height := cBtnBmpSize;
    // Record required colour and size of bmp
    BmpColour := mnuTest.SelectedColour;
    BmpRect := Rect(0, 0, cBtnBmpSize, cBtnBmpSize);
    // Paint border to be taken as transparent: so not bmp colour
    if BmpColour <> clWhite then
      Bmp.Canvas.Brush.Color := clWhite
    else
      Bmp.Canvas.Brush.Color := clBlack;
    Bmp.Canvas.FillRect(BmpRect);
    // Paint a true border
    InflateRect(BmpRect, -1, -1);
    Bmp.Canvas.Brush.Color := clGray;
    Bmp.Canvas.FillRect(BmpRect);
    // Paint bmp colour inside border
    Bmp.Canvas.Brush.Color := BmpColour;
    InflateRect(BmpRect, -1, -1);
    Bmp.Canvas.FillRect(BmpRect);
    // Finally store the glyph
    btnTest.Glyph.Assign(Bmp);
  finally
    Bmp.Free;
  end;
end;

procedure TFmPJColourPopupMenuCE.UpdateControls;
  {Updates enabled property of controls that depend on state of others}
begin
  if mnuTest.DisplayStyle = dsColourOnly then
  begin
    lblBmpHeight.Enabled := True;
    lblBmpWidth.Enabled := True;
    edBmpHeight.Enabled := True;
    edBmpHeight.Color := cmbDisplayStyle.Color;
    edBmpWidth.Enabled := True;
    edBmpWidth.Color := cmbDisplayStyle.Color;
    udBmpHeight.Enabled := True;
    udBmpWidth.Enabled := True;
  end
  else
  begin
    lblBmpHeight.Enabled := False;
    lblBmpWidth.Enabled := False;
    edBmpHeight.Enabled := False;
    edBmpHeight.ParentColor := True;
    edBmpWidth.Enabled := False;
    edBmpWidth.ParentColor := True;
    udBmpHeight.Enabled := False;
    udBmpWidth.Enabled := False;
  end;
end;

end.
